#lang racket/base

(require racket/match)
(require racket/system)
(require racket/string)
(require racket/port)
(require racket/cmdline)
(require logbook)

(define server-variation #f)
(define server-hostname "localhost")

(command-line #:program "external-latency.rkt"
	      #:once-each
	      ["--host" name "hostname where server is reachable" (set! server-hostname name)]
	      #:once-any
	      ["--erlang" "use erlang server" (set! server-variation 'erlang)]
	      ["--minimart" "use minimart server" (set! server-variation 'minimart)]
	      ["--other" name "use other server" (set! server-variation name)])

(when (not server-variation)
  (error 'external-latency "Please choose a server variation."))

(define entry-type (format "external-latency-~a" server-variation))

(define L (default-logbook))
(define E (standard-logbook-entry L "minimart" entry-type))
(define entry-name (logbook-entry-name E))
(define Tmachine (logbook-table E "machine-info" "machine-info"))

(define (start-bg command)
  (match-define (list #f #f pid #f control)
    (process/ports (current-output-port)
		   (current-input-port)
		   (current-error-port)
		   command))
  control)

(log-info "Starting server...")
(define server-control
  (start-bg
   (match server-variation
     ['minimart
      (format "racket echo-server.rkt --logbook-entry-name ~a --logbook-entry-type ~a"
	      entry-name
	      entry-type)]
     ['erlang
      (define erlang-version-command
	"erl -noshell -eval 'io:format(erlang:system_info(otp_release)), halt().'")
      (write-logbook-datum! Tmachine
			    #:label "erlang-version"
			    (string-trim (with-output-to-string
					   (lambda () (system erlang-version-command)))))
      "./run-erlang-server.sh"]
     [(? string? other)
      (printf "Please start the other server on hostname '~a' now.\n" server-hostname)
      (printf "Press enter when it has started.\n")
      (read-line)
      (printf "Continuing...\n")
      "true"
      ])))

(sleep 5)

(log-info "Starting client...")
(define client-control
  (start-bg
   (format "racket echo-client.rkt --host ~a --logbook-entry-name ~a --logbook-entry-type ~a"
	   server-hostname
	   entry-name
	   entry-type)))

(log-info "Waiting for client termination...")
(client-control 'wait)

(log-info "Waiting for server termination...")
(void (match server-variation
	[(? string? other)
	 (printf "Press enter when the server has terminated.\n")
	 (read-line)]))

(server-control 'wait)
(log-info "Done.")
